<?php

/**
 * intiMail main class. Includes all functions necessary to intimail 
 * like authentication, ldap functions...
 *
 * @author Timothée Teneur
 */

class Intimail {

    //////////////////////////////////////////////
    //              INIT VARIABLES              //
    //////////////////////////////////////////////
    
    var $username;
    var $password;
    var $ldapserv;
    var $ldapport;
    var $rand_key;
    var $fqdn;
    var $attachsize;
    
    //////////////////////////////////////////////
    //            CLASS CONSTRUCTION            //
    //////////////////////////////////////////////  
    
    function Intimail(){

        $data = file_get_contents('/var/www/webmail/include/config.inc');
        $data = json_decode($data,true);
        
        $this->username = $data['username'];
        $this->password = $data['password'];
        $this->ldapserv = $data['ldapserv'];
        $this->ldapport = $data['ldapport'];
        $this->rand_key = $data['rand_key'];
        $this->fqdn = $data['fqdn'];
        $this->attachsize = $data['attachsize'];
        
    //////////////////////////////////////////////
    //         OVERRIDE DEFAULT CONFIG          //
    //////////////////////////////////////////////
        
//        $this->username = "admin";
//        $this->password = "ldap";
//        $this->ldapserv = "localhost";
//        $this->ldapport = 389;
//        $this->rand_key = '4HzFMWy2wV8D1v2';
//        $this->fqdn = 'intimail.pw';
//        $this->attachsize = 2000000;
        
    }
    
    //////////////////////////////////////////////
    //              MAIN FUNCTIONS              //
    //////////////////////////////////////////////
    
    //////////////////////////////////////////////
    //             AUTHENTICATION               //
    //////////////////////////////////////////////
    
    function Login(){        
        $username = filter_input(INPUT_POST, 'username');
        $password = filter_input(INPUT_POST, 'password');
        
        if(empty($username)){
            $this->HandleError("Username is empty !");
            return false;
        }
        
        if(empty($password)){
            $this->HandleError("Password is empty !");
            return false;            
        }
        
        if(!isset($_SESSION)){ 
            session_start(); 
        }
        
        if(!$this->CheckLDAPLogin($username,$password)){
            return false;
        }
        
        $_SESSION[$this->GetLoginSessionVar()] = $username;
        
        return true;        
    }
    
    function CheckLDAPLogin($username, $password){
        $ldapconn = ldap_connect($this->ldapserv,$this->ldapport) or die("Could not connect to LDAP server !");
        
        //Si non défini, échec du bind
	ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
        $domain = explode(".",$this->fqdn);
        $dnofuser = "cn=".$username.",dc=people,dc=mail,dc=".$domain[0].",dc=".$domain[1];
        
        if(!(ldap_bind($ldapconn,$dnofuser,$password))){ 
            return false;
        }
        
        $user_data = $this->GetSessionVars($ldapconn,$dnofuser);
        
        $_SESSION['uid'] = $user_data[0][uid][0];
        $_SESSION['mail'] = $user_data[0][mail][0];
        $_SESSION['surname'] = $user_data[0][sn][0];
        $_SESSION['fname'] = $user_data[0][givenname][0]; 
        $_SESSION['fullname'] = $user_data[0][displayname][0];
        $_SESSION['mailbox'] = $user_data[0][mailbox][0];
        $_SESSION['quota'] = $user_data[0][quota][0];
        $_SESSION['homedir'] = $user_data[0][homedirectory][0];
        $_SESSION['dname'] = $user_data[0][dn];
        
        ldap_close($ldapconn);
        
        return true; //<---------------- REMETTRE A TRUE POUR LOGIN, ANTI-RUSSIAN MEASURES !
    }
    
    function CheckLogin(){
        if(!isset($_SESSION)){ 
            session_start(); 
        }
        
        $sessionvar = $this->GetLoginSessionVar();
         
        if(empty($_SESSION[$sessionvar])){
            return false;
        }
    
        return true;
    }
    
    function isAdmin($user){
        $ldapconn = ldap_connect($this->ldapserv,$this->ldapport) or die("Could not connect to LDAP server !");
        ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
        $domain = explode(".",$this->fqdn);
        $admindn = "cn=".$this->username.",dc=".$domain[0].",dc=".$domain[1];
        
        if(!(ldap_bind($ldapconn,$admindn,$this->password))){ 
            echo "Binding failed !";
            return false;                    
        }
        $dn = "cn=admins,dc=groups,dc=mail,dc=intimail,dc=pw";
        $filter = "maildrop=*";
        $result = ldap_search($ldapconn, $dn, $filter);
        $entries = ldap_get_entries($ldapconn, $result);
        $result = explode(",",$entries[0]['maildrop'][0]);
        for($i=0;$i<sizeof($result);$i++){
            if(($user."@".$this->fqdn)==($result[$i])){
                return true;
            }
        }
        return false;
    }
    
    function GetUsersArray(){
        $users = array();
        $ldapconn = ldap_connect($this->ldapserv,$this->ldapport) or die("Could not connect to LDAP server !");
        ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
        $domain = explode(".",$this->fqdn);
        $admindn = "cn=".$this->username.",dc=".$domain[0].",dc=".$domain[1];
        if(!(ldap_bind($ldapconn,$admindn,$this->password))){ 
            echo "Binding failed !";
            return false;                    
        }
        $dn = "dc=people,dc=mail,dc=intimail,dc=pw";
        $filter = "uid=*";
        $result = ldap_search($ldapconn, $dn, $filter);
        $entries = ldap_get_entries($ldapconn, $result);
        for($i=0;$i<sizeof($entries)-1;$i++){
            $temp['uid'] = $entries[$i]['uid'][0];
            $temp['fullname'] = $entries[$i]['displayname'][0];
            $users[$i] = $temp;
        }
        return $users;
    }
    
    function GetListsArray(){
        $users = array();
        $ldapconn = ldap_connect($this->ldapserv,$this->ldapport) or die("Could not connect to LDAP server !");
        ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
        $domain = explode(".",$this->fqdn);
        $admindn = "cn=".$this->username.",dc=".$domain[0].",dc=".$domain[1];
        if(!(ldap_bind($ldapconn,$admindn,$this->password))){ 
            echo "Binding failed !";
            return false;                    
        }
        $dn = "dc=groups,dc=mail,dc=intimail,dc=pw";
        $filter = "uid=*";
        $result = ldap_search($ldapconn, $dn, $filter);
        $entries = ldap_get_entries($ldapconn, $result);
        for($i=0;$i<sizeof($entries)-1;$i++){
            $temp['uid'] = $entries[$i]['uid'][0];
            $users[$i] = $temp;
        }
        return $users;
    }
    
    function ChangeConfig($indice,$value){
        $data = file_get_contents('include/config.inc');
        $data = json_decode($data,true);
        if(($indice==='ldapport')||($indice==='attachsize')){
            $data[$indice] = intval($value);
        }
        else $data[$indice] = $value;
        $data = json_encode($data, JSON_PRETTY_PRINT);
        file_put_contents('include/config.inc',$data);
    }
    
    function ValidatePassword($password, $hash){
        $hash = base64_decode(substr($hash, 6));
        $original_hash = substr($hash, 0, 20);
        $salt = substr($hash, 20);
        $new_hash = mhash(MHASH_SHA1, $password . $salt);
        if (strcmp($original_hash, $new_hash) == 0) echo 'ok';
        else echo 'check failed';
    }

    function GenerateLDAPPass($password){
        $salt = $this->SaltGen();
        $hash = mhash(MHASH_SHA1,$password.$salt);
        $hash = base64_encode($hash.$salt);
        return "{SSHA}".$hash;
    }
    
    function GetLoginSessionVar(){
        $retvar = md5($this->rand_key);
        $retvar = 'usr_'.substr($retvar,0,10);
        return $retvar;
    }
    
    function GetSessionVars($ldapconn,$dnofuser){
        $search_filter = "(objectClass=*)";
        $result = ldap_read($ldapconn, $dnofuser, $search_filter) or die('Could not perform read !');
        $entries = ldap_get_entries($ldapconn, $result);
        return $entries;        
    }
    
    function LogOut(){
        session_start();
        $sessionvar = $this->GetLoginSessionVar();
        $_SESSION[$sessionvar]=NULL;
        unset($_SESSION[$sessionvar]);
    }
    
    function AddUser($firstname,$lastname,$password,$quota){
        $user = array();
        $ldapconn = ldap_connect($this->ldapserv,$this->ldapport) or die("Could not connect to LDAP server !");
        ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
        $domain = explode(".",$this->fqdn);
        $admindn = "cn=".$this->username.",dc=".$domain[0].",dc=".$domain[1];
        if(!(ldap_bind($ldapconn,$admindn,$this->password))){ 
            echo "Binding failed !";
            return false;                    
        }
        $uid = strtolower($firstname[0]).strtolower($lastname);
        $mail = $uid."@".$this->fqdn;
        $name = $firstname." ".$lastname;
        $mailbox = $this->fqdn."/".$uid."/";
        $dn = "cn=".$uid.",dc=people,dc=mail,dc=intimail,dc=pw";
        $quota = intval($quota);
        $password = $this->GenerateLDAPPass($password);
        
        //Filling user array
        $user['uid'] = $uid;
        $user['mail'] = $mail;
        $user['sn'] =  $lastname;
        $user['givenName'] = $firstname;
        $user['displayName'] = $firstname." ".$lastname;
        $user['mailbox'] = $mailbox;
        $user['homeDirectory'] = "/home/vmail/";
        $user['objectClass'][0] = "top";
        $user['objectClass'][1] = "inetOrgPerson";
        $user['objectClass'][2] = "CourierMailAccount";
        $user['cn'] = $uid;
        $user['userPassword'] = $password;
        $user['quota'] = $quota;
        
        if(!(ldap_add($ldapconn, $dn, $user))){
            echo "Failed to add user to LDAP ! Either you supplied wrong parameters or someone with that UID already exists.";
            return false;
        }
        else{
            echo "Successfully added user ".$user['displayName']." (".$uid.") to LDAP !";
            
            $txt = "Hello ".$user['givenName']." !\r\n\r\n Make good use of your new mailbox, and enjoy privacy on the internet, again !\r\n\r\nThe intiMail Team.";
            $headers = "From: postmaster@".$this->fqdn;
            $enveloppe = '-fpostmaster@'.$this->fqdn;
            if(!(mail($user['mail'],"Welcome to your private mailbox !",$txt,$headers,$enveloppe))){
                echo " But failed sending welcome mail !";
            }
            else{ 
                echo " Also, successfully sent welcome mail !";
                $count = 0;
                while(!((is_dir('/var/www/webmail/userdata/'.$user['mailbox']))||($count>=20))){
                    sleep(3);
                    $count++;
                }
                mkdir('/var/www/webmail/userdata/'.$user['mailbox']."sent/",0770,true);
                file_put_contents('/var/www/webmail/userdata/'.$user['mailbox']."json/sent.json",'['.PHP_EOL.']');
                $this->AddUserToList($user['uid'], 'all');
            }
            return true;
        }
    }
    
    function DeleteUser($uid){
        $ldapconn = ldap_connect($this->ldapserv,$this->ldapport) or die("Could not connect to LDAP server !");
        ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
        $domain = explode(".",$this->fqdn);
        $admindn = "cn=".$this->username.",dc=".$domain[0].",dc=".$domain[1];
        if(!(ldap_bind($ldapconn,$admindn,$this->password))){ 
            echo "Binding failed !";
            return false;                    
        }
        $dn = "cn=".$uid.",dc=people,dc=mail,dc=intimail,dc=pw";
        if(!(ldap_delete($ldapconn, $dn))){
            echo "Failed to remove user from LDAP ! Either you supplied a wrong UID or that person does not exist.";
            return false;
        }
        else{
            echo "Successfully removed user ".$uid." from LDAP !";
            $path = "/var/www/webmail/userdata/".$this->fqdn."/".$uid."/";
            exec("rm -rf {$path}");
            $list = $this->GetListsArray();
            for($i=0;$i<sizeof($list);$i++){
                $this->RemoveUserFromList($uid, $list[$i]['uid']);
            }
            return true;
        }
    }
    
    function ModifyUser($uid,$field,$value){
        $ldapconn = ldap_connect($this->ldapserv,$this->ldapport) or die("Could not connect to LDAP server !");
        ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
        $domain = explode(".",$this->fqdn);
        $admindn = "cn=".$this->username.",dc=".$domain[0].",dc=".$domain[1];
        if(!(ldap_bind($ldapconn,$admindn,$this->password))){ 
            echo "Binding failed !";
            return false;                    
        }
        $dn = "cn=".$uid.",dc=people,dc=mail,dc=intimail,dc=pw";
        
        if($field==='name'){
            $exploded = explode(' ',$value);
            $entry['givenName'] = $exploded[0];
            $entry['sn'] = $exploded[1];
            $entry['displayName'] = $value;
        }
        elseif($field==='quota'){
            $entry['quota'] = $value;
        }
        elseif($field==='password'){
            $entry['userPassword'] = $this->GenerateLDAPPass($value);
        }
        elseif($field==='uid'){
            if(ldap_rename($ldapconn,$dn,"cn=".$value,NULL,true)){
                $entry['uid'] = $value;
                $entry['mail'] = $value.'@'.$this->fqdn;
                $entry['mailbox'] = $this->fqdn.'/'.$value."/";
                $dn = "cn=".$value.",dc=people,dc=mail,dc=intimail,dc=pw";
                $oldpath = "/var/www/webmail/userdata/".$this->fqdn."/".$uid."/";
                $newpath = "/var/www/webmail/userdata/".$this->fqdn."/".$value."/";
                $result = exec("mv {$oldpath} {$newpath}");
            }
        }
        else{ 
            echo "Invalid field name supplied while changing user attributes !"; 
        }
        
        ldap_modify($ldapconn, $dn, $entry);      
    }
    
    function GetUsersFromList($list){
        $ldapconn = ldap_connect($this->ldapserv,$this->ldapport) or die("Could not connect to LDAP server !");
        ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
        $domain = explode(".",$this->fqdn);
        $admindn = "cn=".$this->username.",dc=".$domain[0].",dc=".$domain[1];
        if(!(ldap_bind($ldapconn,$admindn,$this->password))){ 
            echo "Binding failed !";
            return false;                    
        }
        $dn = "cn=".$list.",dc=groups,dc=mail,dc=intimail,dc=pw";
        $filter = "maildrop=*";
        $result = ldap_search($ldapconn, $dn, $filter);
        $entries = ldap_get_entries($ldapconn, $result);
        $adresses = $entries[0]['maildrop'][0];
        $adresses = explode(',',$adresses);
        return $adresses;
    }
    
    function AddUserToList($uid,$list){
        $ldapconn = ldap_connect($this->ldapserv,$this->ldapport) or die("Could not connect to LDAP server !");
        ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
        $domain = explode(".",$this->fqdn);
        $admindn = "cn=".$this->username.",dc=".$domain[0].",dc=".$domain[1];
        if(!(ldap_bind($ldapconn,$admindn,$this->password))){ 
            echo "Binding failed !";
            return false;                    
        }
        $members = $this->GetUsersFromList($list);
        $new_user = $uid.'@'.$this->fqdn;
        array_push($members,$new_user);
        $members = implode(',',$members);
        $entry['maildrop'] = $members;
        $dn = "cn=".$list.",dc=groups,dc=mail,dc=intimail,dc=pw";
        return ldap_modify($ldapconn, $dn, $entry);
    }
    
    function RemoveUserFromList($uid,$list){
        $ldapconn = ldap_connect($this->ldapserv,$this->ldapport) or die("Could not connect to LDAP server !");
        ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
        $domain = explode(".",$this->fqdn);
        $admindn = "cn=".$this->username.",dc=".$domain[0].",dc=".$domain[1];
        if(!(ldap_bind($ldapconn,$admindn,$this->password))){ 
            echo "Binding failed !";
            return false;                    
        }
        $members = $this->GetUsersFromList($list);
        $user = $uid.'@'.$this->fqdn;
        $members = array_merge(array_diff($members, array($user)));
        $members = implode(',',$members);
        $entry['maildrop'] = $members;
        $dn = "cn=".$list.",dc=groups,dc=mail,dc=intimail,dc=pw";
        return ldap_modify($ldapconn, $dn, $entry);
    }
        
    function AddGroup($groupname){
        $user = array();
        $ldapconn = ldap_connect($this->ldapserv,$this->ldapport) or die("Could not connect to LDAP server !");
        ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
        $domain = explode(".",$this->fqdn);
        $admindn = "cn=".$this->username.",dc=".$domain[0].",dc=".$domain[1];
        if(!(ldap_bind($ldapconn,$admindn,$this->password))){ 
            echo "Binding failed !";
            return false;                    
        }
        $dn = "cn=".$groupname.",dc=groups,dc=mail,dc=intimail,dc=pw";
        
        //Filling group array
        $user['uid'] = $groupname;
        $user['mail'] = $groupname."@".$this->fqdn;
        $user['sn'] =  $groupname;
        $user['objectClass'][0] = "top";
        $user['objectClass'][1] = "inetOrgPerson";
        $user['objectClass'][2] = "CourierMailAlias";
        $user['cn'] = $groupname;
        $user['maildrop'] = "postmaster@".$this->fqdn;
        
        if(!(ldap_add($ldapconn, $dn, $user))){
            echo "Failed to add group to LDAP ! Either you supplied wrong parameters or this group already exists.";
            return false;
        }
        else{
            echo "Successfully added this group to LDAP !";
            return true;
        }
    }
    
    function DeleteGroup($groupname){
        $ldapconn = ldap_connect($this->ldapserv,$this->ldapport) or die("Could not connect to LDAP server !");
        ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
        $domain = explode(".",$this->fqdn);
        $admindn = "cn=".$this->username.",dc=".$domain[0].",dc=".$domain[1];
        if(!(ldap_bind($ldapconn,$admindn,$this->password))){ 
            echo "Binding failed !";
            return false;                    
        }
        $dn = "cn=".$groupname.",dc=groups,dc=mail,dc=intimail,dc=pw";
        if(!(ldap_delete($ldapconn, $dn))){
            echo "Failed to remove this group from LDAP ! Either you supplied a wrong GID or that group does not exist.";
            return false;
        }
        else{
            echo "Successfully removed this group from LDAP !";
            return true;
        }
    }
    
    ///////////////////////////////////////////////
    //         MISC OTHER STUFF FOR NOW          //
    ///////////////////////////////////////////////
    
    function HandleError($err){
        $this->error_message .= $err."\r\n";
    }
    
    function GetSelfScript(){
        return htmlentities($_SERVER['PHP_SELF']);
    }  
    
    function CurrentQuota($domain){
        $output = exec('cat userdata/'.$domain.'json/quota.json');
        $output = intval($output)/1000000;
        $output = round($output,2);
        return $output;
    }
    
    function CurrentQuotaRatio(){
        $current = $this->CurrentQuota($_SESSION['mailbox']);
        $max = filter_var($_SESSION['quota'], FILTER_SANITIZE_NUMBER_INT);
        $max=$max/1000000;
        $ratio = round($current/$max,2)*100;
        echo $ratio;
    }
    
    function QIDGen(){
        $characters = '0123456789ABCDEF';
         $string = '';
         for ($i = 0; $i < 10; $i++) {
            $string .= $characters[rand(0, strlen($characters) - 1)];
         }
         return $string;
    }
    
    function SaltGen(){
        $characters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789/\\][{}\'";:?.>,<!@#$%^&*()-_=+|';
        $string = '';
        for ($i = 0; $i < 4; $i++) {
            $string .= $characters[rand(0, strlen($characters) - 1)];
        }
        return $string;
    }
    
    function RemoveJson($id,$json,$mailbox){
        $path = 'userdata/'.$mailbox.'json/'.$json;
        $data = file_get_contents($path);
        $data = json_decode($data,true);
        foreach($data as $k => $v){
            if($v['id']==$id){
                $ind=$k;
                unset($data[$k]);
            }
        }
        $data = json_encode(array_values($data),JSON_PRETTY_PRINT);
        file_put_contents('userdata/'.$mailbox.'/json/'.$json,$data);
    }
    
    function AddJson($json,$mailbox,$mail){
        $path = 'userdata/'.$mailbox.'json/'.$json;
        $data = file_get_contents($path);
        $data = json_decode($data,true);
        $id = $data[0]['id'];
        //Got latest ID, writting new header
        $mail['id']= strval($id+1);
        //Putting new mail header in the array
        array_unshift($data,$mail);
        //Reconstructing json file
        $data = json_encode(array_values($data),JSON_PRETTY_PRINT);
        file_put_contents('userdata/'.$mailbox.'/json/'.$json,$data);        
    }
    
    function UpdateQuota($size,$mailbox){
        $path = 'userdata/'.$mailbox.'/json/quota.json';
        echo $path;
        $current = file_get_contents($path);
        echo $current;
        $current = intval($current)+(int)$size;
        file_put_contents($path,$current);
    }
    
    function AddMail($msg,$subject,$to,$from,$attach,$mailbox){
        $time = time();
        $date = date('Y-m-d H:i:s',$time);
        $queueid = $this->QIDGen();
        $path = 'userdata/'.$mailbox.'sent/'.$time.'.'.$queueid;
        $msg = explode('; charset=UTF-8',$msg);
        file_put_contents($path,$msg[1]);
        $size = strval(filesize($path));
        $header = [ 
            "from" => $from, 
            "subject" => $subject, 
            "timestamp" => $date, 
            "unixtimestamp" => strval($time), 
            "queueid" => $queueid, 
            "size" => $size,
            "status" => "0",
            "pj" => $attach,
            "id" => "0",
            "to" => $to 
            ];
        $this->AddJson('sent.json',$mailbox,$header);
        $this->UpdateQuota(intval($size),$mailbox);
    }
    
    function RemoveMail(){}
    
    function Test(){
        echo "This is a test function";
    }
    
}

$intimail = new Intimail();

//$data = file_get_contents('include/config.json');
//echo $data;

?>
